#include <windows.h>	// We Need The Windows Header
#include <WinUser.h> //contains window manager msg defines

#include "stb_image.h"
#include "psnprintf.h"


int main(int argc, char **argv)
{
   int w = 0;
   int h = 0;
   int n = 0;
   unsigned char* data = 0;
   FILE *output = fopen("eghfont.h", "w");

   //load screenshot png file
   data = stbi_load("font.png", &w, &h, &n, 4);
   
   if(!data)
   {
      printf("ERROR: could not open file");
      return 1;
   }

   if(h != 8)
   {
      printf("ERROR: Unexpected image height %d\n",h);
      return 1;
   }

   //file header
   fputs("const unsigned char font[] PROGMEM = \r\n", output);
   fputs("{\r\n", output);
   fputs("   0x00, 0x00, 0x00, 0x00, 0x00, // space\r\n", output);

   //font is made up of vertical slices, 1 byte each
   char currChar = '!';
   int currByte = 0;
   char buff[64];
   unsigned char byte[5] = {0,0,0,0,0};
   for(int x = 0; x < w; x++)
   {
      //scan down and build up a byte
      for (int y = 0; y < 8; y++)
      {
         if(data[(x * 4) + (y * w * 4)] )
         {
            byte[currByte] |= (1 << y);
         }
      }

      //are we done collecting slices for this character?
      // special case for quote since it has a gap in the middle
      // otherwise just check when we reach the end
      bool glyphDone = false;
      if(currChar == '"')
      {
         if(currByte >= 3)
            glyphDone = true;
      }
      else if(currByte >= 4 && byte[currByte] != 0x00)
      {
         //wide glyph that had to use all 5 slices
         glyphDone = true;

         //skip over the space next to us
         x++;
      }
      else if(byte[currByte] == 0x00)
      {
         glyphDone = true;
      }
      
      if(glyphDone)
      {
         psnprintf(buff, sizeof(buff), "   0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, // %c\r\n", byte[0], byte[1], byte[2], byte[3], byte[4], currChar);
         fputs(buff, output);

         byte[0] = 0;
         byte[1] = 0;
         byte[2] = 0;
         byte[3] = 0;
         byte[4] = 0;
         currByte = 0;

         if(currChar >= '~') //degrees symbol
         {
            break;
         }
         currChar++;                  
         continue;
      }

      currByte++;
   }

   //file footer
   fputs("};\r\n", output);
   fputs("\r\n", output);   
   fputs("#define INSTRUCTION_MASK 0xC0\r\n", output);
   fputs("#define _I (1<<6)\r\n", output);
   fputs("#define _0 (0<<6)\r\n", output);
   fputs("#define _F (3<<6)\r\n", output);
   fputs("#define _S (2<<6)\r\n", output);
   fputs("\r\n", output);
   fputs("const unsigned char * const * const fTbl[] PROGMEM =\r\n", output);
   fputs("{\r\n", output);
   fputs("   0,0,0,0,0,0,0\r\n", output);
   fputs("};\r\n", output);

   //done
   fclose(output);
   
	return 0;
}

